/*
 ---

 script: Modal.js

 description: MUI.Modal - Create modal dialog windows.

 copyright: (c) 2011 Contributors in (/AUTHORS.txt).

 license: MIT-style license in (/MIT-LICENSE.txt).

 See Also: <Window>

 requires:
 - MochaUI/MUI
 - MochaUI/MUI.Window

 provides: [MUI.Modal]

 ...
 */

define('mocha/Controls/modal/modal', [

    'mocha/Controls/window/window'

], function(MUI_Window)
{
    MUI.Modal = (MUI.Modal || new NamedClass('MUI.Modal', {}));

    MUI.Modal.implement({

        Extends: MUI_Window,

        options: {
            type: 'modal'
        },

        initialize: function(options)
        {
            if (!options.type) {
                options.type='modal';
            }

            if ( !$('modalOverlay') )
            {
                this._modalInitialize();

                window.addEvent('resize', function(){
                    this._setModalSize();
                }.bind(this));
            } else
            {
                $('modalOverlay').setStyle('display', 'inline');
            }

            this.parent( options );
        },

        _modalInitialize: function()
        {
            var modalOverlay = new Element('div', {
                id     : 'modalOverlay',
                styles : {
                    height  : document.getCoordinates().height,
                    opacity : 0.6,
                    display : 'inline'
                }
            }).inject( document.body );

            modalOverlay.setStyles({
                position : Browser.ie6 ? 'absolute' : 'fixed'
            });

            modalOverlay.addEvent('click', function()
            {
                var instance = MUI.get(MUI.currentModal.id);

                if (instance && instance.options.modalOverlayClose)
                {
                    MUI.currentModal.close();

                    (function()
                    {
                        var highest_win = MUI.Windows._getWithHighestZIndex();

                        if (highest_win && highest_win.hasClass('modal')) {
                            MUI.currentModal = highest_win;
                        }
                    }).delay(200);
                }
            });

            if ( Browser.ie6 )
            {
                var modalFix = new Element('iframe', {
                    'id'           : 'modalFix',
                    'scrolling'    : 'no',
                    'marginWidth'  : 0,
                    'marginHeight' : 0,
                    'src'          : '',
                    'styles'       : {
                        'height' : document.getCoordinates().height
                    }
                }).inject(document.body);
            }

            /*
            MUI.Modal.modalOverlayOpenMorph = new Fx.Morph($('modalOverlay'), {
                'duration': 150
            });
            */

            MUI.Modal.modalOverlayOpenMorph = moofx( $('modalOverlay') );

            /*
            MUI.Modal.modalOverlayCloseMorph = new Fx.Morph($('modalOverlay'), {
                'duration': 150,
                onComplete: function(){
                    $('modalOverlay').hide();
                    if (Browser.ie6){
                        $('modalFix').hide();
                    }
                }.bind(this)
            });
            */

            MUI.Modal.modalOverlayCloseMorph = moofx( $('modalOverlay') );
        },

        _setModalSize: function()
        {
            $('modalOverlay').setStyle('height', document.getCoordinates().height);

            if (Browser.ie6) {
                $('modalFix').setStyle('height', document.getCoordinates().height);
            }
        }

    });

    return MUI.Modal;
});

